#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This file holds a class that can run simulation jobs in parallel.
# Written by Andreas Nicolai for DELPHIN Simulation job processing.

import subprocess		# import the module for calling external programs (creating subprocesses)
import sys
import os
import shutil
import threading,Queue

class JobRunner:
	"""
	This class encapsulates a thread queue, a runner function and provides a convenient
	interface for running jobs in parallel.
	"""
	def __init__(self, N):
		# create a queue for task scheduling
		self.jobQueue = Queue.Queue()
		self.N = N

	# here we define our solver thread function
	def thread_function(self):
		"""This function runs a simulation solver."""
	
		# run as long as we have jobs in the Queue
		while 1:
			# get a new task from the Queue
			job = self.jobQueue.get()
			print("Running '{}'".format(job[-1])) # print project file, should be last argument
			# CREATE_NEW_CONSOLE is needed for Windows
			try:
				if sys.platform == "win32":
					solverProcess = subprocess.Popen(job,creationflags=subprocess.CREATE_NEW_CONSOLE) 
				else:
					solverProcess = subprocess.Popen(job)
				solverProcess.wait()
			except Exception as e:
				print(str(e))
			# tell the Queue that the task was done
			self.jobQueue.task_done()

	def run(self, jobs):
		# now create as many threads as we need
		for i in range(self.N):
			t = threading.Thread(target=JobRunner.thread_function,args=[self])
			t.daemon = True
			t.start()
		# now add project files to the queue
		for job in jobs:
			self.jobQueue.put(job)
		# wait until all tasks are done
		self.jobQueue.join()
		return
